@@ -26,6 +26,7 @@
 #include <atalk/fce_api.h>
 #include <atalk/netatalk_conf.h>
 #include <atalk/spotlight.h>
+#include <atalk/dsi.h>
 
 #include "directory.h"
 #include "dircache.h"
@@ -252,7 +253,7 @@ restart:
                     }
                     LOG(log_error, logtype_afpd, "Reopen volume %s using in memory temporary CNID DB.",
                         vol->v_path);
-                    vol->v_cdb = cnid_open(vol->v_path, vol->v_umask, "tdb", flags, NULL, NULL, NULL, NULL);
+                    vol->v_cdb = cnid_open(vol, "tdb", flags);
                     if (vol->v_cdb) {
                         if (!(vol->v_flags & AFPVOL_TM)) {
                             vol->v_flags |= AFPVOL_RO;
@@ -764,8 +765,7 @@ int afp_createfile(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf _U_,
 createfile_iderr:
     ad_flush(&ad);
     ad_close(&ad, ADFLAGS_DF|ADFLAGS_HF );
-    fce_register(FCE_FILE_CREATE, fullpathname(upath), NULL, fce_file);
-    sl_index_file(path);
+    fce_register(obj, FCE_FILE_CREATE, fullpathname(upath), NULL);
 
     curdir->d_offcnt++;
     setvoltime(obj, vol );
@@ -1449,7 +1449,9 @@ int copyfile(struct vol *s_vol,
         return AFPERR_EXIST;
     }
 
-    if ((err = copy_fork(ADEID_DFORK, &add, adp)) != 0)
+    if ((err = copy_fork(ADEID_DFORK, &add, adp,
+                         s_vol->v_obj->dsi->commands,
+                         s_vol->v_obj->dsi->server_quantum)) != 0)
         LOG(log_error, logtype_afpd, "copyfile('%s'): %s", src, strerror(errno));
 
     if (err == 0)
@@ -1662,7 +1664,7 @@ int afp_createid(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf, si
         return( AFPERR_PARAM);
     }
 
-    if (vol->v_cdb == NULL || !(vol->v_cdb->flags & CNID_FLAG_PERSISTENT)) {
+    if (vol->v_cdb == NULL || !(vol->v_cdb->cnid_db_flags & CNID_FLAG_PERSISTENT)) {
         return AFPERR_NOOP;
     }
 
@@ -1801,7 +1803,7 @@ int afp_resolveid(AFPObj *obj, char *ibuf, size_t ibuflen _U_, char *rbuf, size_
         return( AFPERR_PARAM);
     }
 
-    if (vol->v_cdb == NULL || !(vol->v_cdb->flags & CNID_FLAG_PERSISTENT)) {
+    if (vol->v_cdb == NULL || !(vol->v_cdb->cnid_db_flags & CNID_FLAG_PERSISTENT)) {
         return AFPERR_NOOP;
     }
 
@@ -1909,7 +1911,7 @@ int afp_deleteid(AFPObj *obj _U_, char *ibuf, size_t ibuflen _U_, char *rbuf _U_
         return( AFPERR_PARAM);
     }
 
-    if (vol->v_cdb == NULL || !(vol->v_cdb->flags & CNID_FLAG_PERSISTENT)) {
+    if (vol->v_cdb == NULL || !(vol->v_cdb->cnid_db_flags & CNID_FLAG_PERSISTENT)) {
         return AFPERR_NOOP;
     }
 
