@@ -68,12 +68,20 @@ int sys_get_easize(VFS_FUNC_ARGS_EA_GETSIZE)
 
     LOG(log_debug7, logtype_afpd, "sys_getextattr_size(%s): attribute: \"%s\"", uname, attruname);
 
-    if ((oflag & O_NOFOLLOW) ) {
-        ret = sys_lgetxattr(uname, attruname, rbuf +4, 0);
-    }
-    else {
-        ret = sys_getxattr(uname, attruname,  rbuf +4, 0);
+    /* PBaranski fix */
+    if (fd != -1) {
+	LOG(log_debug, logtype_afpd, "sys_get_easize(%s): file is already opened", uname);
+	ret = sys_fgetxattr(fd, attruname, rbuf +4, 0);
+    } else {
+	if ((oflag & O_NOFOLLOW) ) {
+    	    ret = sys_lgetxattr(uname, attruname, rbuf +4, 0);
+	}
+	else {
+    	    ret = sys_getxattr(uname, attruname,  rbuf +4, 0);
+	}
     }
+    /* PBaranski fix */
+
     
     if (ret == -1) {
         memset(rbuf, 0, 4);
@@ -99,6 +107,20 @@ int sys_get_easize(VFS_FUNC_ARGS_EA_GETSIZE)
     if (ret > MAX_EA_SIZE) 
       ret = MAX_EA_SIZE;
 
+    if (vol->v_flags & AFPVOL_EA_SAMBA) {
+        /* What can we do about xattrs that are 1 byte large? */
+        if (ret < 2) {
+            memset(rbuf, 0, 4);
+            *rbuflen += 4;
+
+            if (vol->v_obj->afp_version >= 34) {
+                return AFPERR_NOITEM;
+            }
+            return AFPERR_MISC;
+        }
+        ret--;
+    }
+
     /* Start building reply packet */
     LOG(log_debug7, logtype_afpd, "sys_getextattr_size(%s): attribute: \"%s\", size: %u", uname, attruname, ret);
 
@@ -125,7 +147,7 @@ int sys_get_easize(VFS_FUNC_ARGS_EA_GETSIZE)
  *    oflag        (r) link and create flag
  *    attruname    (r) name of attribute
  *    maxreply     (r) maximum EA size as of current specs/real-life
- *
+ *    fd           (r) file descriptor
  * Returns: AFP code: AFP_OK on success or appropiate AFP error code
  *
  * Effects:
@@ -145,14 +167,24 @@ int sys_get_eacontent(VFS_FUNC_ARGS_EA_GETCONTENT)
         maxreply = MAX_EA_SIZE;
 
     LOG(log_debug7, logtype_afpd, "sys_getextattr_content(%s): attribute: \"%s\", size: %u", uname, attruname, maxreply);
+    if (vol->v_flags & AFPVOL_EA_SAMBA) {
+        maxreply++;
+    }
 
-    if ((oflag & O_NOFOLLOW) ) {
-        ret = sys_lgetxattr(uname, attruname, rbuf +4, maxreply);
+    /* PBaranski fix */
+    if (fd != -1) {
+	LOG(log_debug, logtype_afpd, "sys_get_eacontent(%s): file is already opened", uname);
+	ret = sys_fgetxattr(fd, attruname, rbuf +4, maxreply);
+    } else {
+	if ((oflag & O_NOFOLLOW) ) {
+    	    ret = sys_lgetxattr(uname, attruname, rbuf +4, maxreply);
+	}
+	else {
+    	    ret = sys_getxattr(uname, attruname,  rbuf +4, maxreply);
+	}
     }
-    else {
-        ret = sys_getxattr(uname, attruname,  rbuf +4, maxreply);
-    }
-    
+    /* PBaranski fix */
+
     if (ret == -1) {
         memset(rbuf, 0, 4);
         *rbuflen += 4;
@@ -173,6 +205,20 @@ int sys_get_eacontent(VFS_FUNC_ARGS_EA_GETCONTENT)
         }
     }
 
+    if (vol->v_flags & AFPVOL_EA_SAMBA) {
+        /* What can we do about xattrs that are 1 byte large? */
+        if (ret < 2) {
+            memset(rbuf, 0, 4);
+            *rbuflen += 4;
+
+            if (vol->v_obj->afp_version >= 34) {
+                return AFPERR_NOITEM;
+            }
+            return AFPERR_MISC;
+        }
+        ret--;
+    }
+
     /* remember where we must store length of attribute data in rbuf */
     *rbuflen += 4 +ret;
 
@@ -210,17 +256,25 @@ int sys_list_eas(VFS_FUNC_ARGS_EA_LIST)
     int     ret, len, nlen;
     char    *buf;
     char    *ptr;
-        
+    struct adouble ad, *adp;
+
     buf = malloc(ATTRNAMEBUFSIZ);
     if (!buf)
         return AFPERR_MISC;
 
-    if ((oflag & O_NOFOLLOW)) {
-        ret = sys_llistxattr(uname, buf, ATTRNAMEBUFSIZ);
-    }
-    else {
-        ret = sys_listxattr(uname, buf, ATTRNAMEBUFSIZ);
+    /* PBaranski fix */
+    if ( fd != -1) {
+	LOG(log_debug, logtype_afpd, "sys_list_eas(%s): file is already opened", uname);
+	ret = sys_flistxattr(fd, uname, buf, ATTRNAMEBUFSIZ);
+    } else {
+	if ((oflag & O_NOFOLLOW)) {
+    	    ret = sys_llistxattr(uname, buf, ATTRNAMEBUFSIZ);
+	}
+	else {
+    	    ret = sys_listxattr(uname, buf, ATTRNAMEBUFSIZ);
+	}
     }
+    /* PBaranski fix */
 
     if (ret == -1) switch(errno) {
         case OPEN_NOFOLLOW_ERRNO:
@@ -291,20 +345,42 @@ int sys_set_ea(VFS_FUNC_ARGS_EA_SET)
 {
     int attr_flag;
     int ret;
+    char *eabuf;
+
+    /*
+     * Buffer for a copy of the xattr plus one byte in case
+     * AFPVOL_EA_SAMBA is used
+     */
+    eabuf = malloc(attrsize + 1);
+    if (eabuf == NULL) {
+        return AFPERR_MISC;
+    }
+    memcpy(eabuf, ibuf, attrsize);
+    eabuf[attrsize] = 0;
 
     attr_flag = 0;
     if ((oflag & O_CREAT) ) 
         attr_flag |= XATTR_CREATE;
-
     else if ((oflag & O_TRUNC) ) 
         attr_flag |= XATTR_REPLACE;
-    
-    if ((oflag & O_NOFOLLOW) ) {
-        ret = sys_lsetxattr(uname, attruname,  ibuf, attrsize,attr_flag);
+
+    if (vol->v_flags & AFPVOL_EA_SAMBA) {
+        attrsize++;
     }
-    else {
-        ret = sys_setxattr(uname, attruname,  ibuf, attrsize, attr_flag);
+
+    /* PBaranski fix */
+    if ( fd != -1) {
+	LOG(log_debug, logtype_afpd, "sys_set_ea(%s): file is already opened", uname);
+	ret = sys_fsetxattr(fd, attruname, eabuf, attrsize, attr_flag);
+    } else {
+	if ((oflag & O_NOFOLLOW) ) {
+   	    ret = sys_lsetxattr(uname, attruname, eabuf, attrsize,attr_flag);
+	}
+	else {
+   	    ret = sys_setxattr(uname, attruname, eabuf, attrsize, attr_flag);
+	}
     }
+    /* PBaranski fix */
 
     if (ret == -1) {
         switch(errno) {
@@ -347,6 +423,7 @@ int sys_set_ea(VFS_FUNC_ARGS_EA_SET)
  *    uname        (r) filename
  *    attruname    (r) EA name
  *    oflag        (r) link and create flag
+ *    fd           (r) file descriptor
  *
  * Returns: AFP code: AFP_OK on success or appropiate AFP error code
  *
@@ -358,12 +435,19 @@ int sys_remove_ea(VFS_FUNC_ARGS_EA_REMOVE)
 {
     int ret;
 
-    if ((oflag & O_NOFOLLOW) ) {
-        ret = sys_lremovexattr(uname, attruname);
-    }
-    else {
-        ret = sys_removexattr(uname, attruname);
+    /* PBaranski fix */
+    if ( fd != -1) {
+	LOG(log_debug, logtype_afpd, "sys_remove_ea(%s): file is already opened", uname);
+	ret = sys_fremovexattr(fd, uname, attruname);
+    } else {
+	if ((oflag & O_NOFOLLOW) ) {
+    	    ret = sys_lremovexattr(uname, attruname);
+	}
+	else {
+    	    ret = sys_removexattr(uname, attruname);
+	}
     }
+    /* PBaranski fix */
 
     if (ret == -1) {
         switch(errno) {
